
/*******************
 * From src: autodiscovery/src/js/autodiscovery-notification.js
 *******************/
$(document).ready(function() {
    var urlPath = "/monitor/index.php/autodiscovery";
    var username = (_user && _user.username) || "";
    var cacheKey = "autodiscoveryNotifications:" + username;
    var cacheKeyTimestamp = "autodiscoveryNotificationsTimestamp:" + username;
    var cacheGracePeriod = 5 * 1000; // Don't fetch if cache age <= 5 s
    var cacheIgnorePeriod = 60 * 60 * 1000; // Don't show if cache age > 1 h

    /**
     *  Show nagbar notifications about scheduled discoveries.
     *
     *  When a scheduled autodiscovery execution finds host addresses that are
     *  not yet in Monitor this will show a nagbar notification with a link to
     *  the scheduled execution.
     *
     * Because the API request can take quite long time to complete, the
     * notifications will be cached. If the cached notifications are not too
     * old they are shown before a new result is ready. If the cached
     * notifications are recent enough a new result will not even be
     * requested at this time.
     */
    function notifyScheduledResults() {
        if (!checkCachedResults()) {
            checkScheduledResults();
        }
    }

    /**
     *  Show cached notifications, if any.
     *
     *  @return {boolean} Whether the cached results were recent enough to
     *                    not warrant a request for new data.
     */
    function checkCachedResults() {
        var cacheTimestamp = parseInt(localStorage.getItem(cacheKeyTimestamp), 10);
        if (cacheTimestamp && !isNaN(cacheTimestamp)) {
            var timeDiff = Date.now() - cacheTimestamp;
            if (timeDiff >= 0 && timeDiff <= cacheIgnorePeriod) {
                var cachedNotifications = localStorage.getItem(cacheKey);
                if (cachedNotifications) {
                    JSON.parse(cachedNotifications).forEach(function(msg) {
                        var notification = showNotification(msg);
                        notification.element.addClass("cached");
                    });
                }
            }
            return timeDiff >= 0 && timeDiff <= cacheGracePeriod;
        }
        return false;
    }

    /**
     *  Fetch and show scheduled results from Magellan.
     */
    function checkScheduledResults() {
        localStorage.removeItem(cacheKey);
        localStorage.removeItem(cacheKeyTimestamp);
        $.ajax({
            url : (
                '/api/magellan/v1/scheduled-results' +
                '?auth_method=ninja' +
                '&username=' + username
            ),
            type : 'GET',
            async: true,
            success : function(response) {
                response.forEach(function(execution) {
                    var count = execution.unmonitored.length;
                    if (count > 0) {
                        var url = urlPath + '#/discoveries/' + execution.id;
                        var msg = (
                            '<a href="' + url + '">' +
                            count + ' unmonitored host' +
                            (count == 1 ? '' : 's') + ' discovered</a>'
                        );
                        clearCachedNotifications();
                        showNotification(msg);
                        cacheNotification(msg);
                    }
                });
            }
        });
    }

    /**
     *  Clear all nagbar notifications that were created from cache.
     */
    function clearCachedNotifications() {
        $(".notify-notification-bar .cached").remove();
    }

    /**
     *  Cache a notification in localStorage.
     *
     *  @param {string} msg The notification message to cache.
     */
    function cacheNotification(msg) {
        var cachedNotifications = JSON.parse(localStorage.getItem(cacheKey) || "[]");
        cachedNotifications.push(msg);
        localStorage.setItem(cacheKey, JSON.stringify(cachedNotifications));
        localStorage.setItem(cacheKeyTimestamp, Date.now());
    }

    /**
     *  Show the notification as a nagbar in Ninja.
     *
     *  @param {string} msg The notification message to show.
     *  @return {object} The created jQuery element.
     *
     */
    function showNotification(msg) {
        // This hack adds a dummy parameter to the link URL in cases when the
        // link would otherwise not cause the browser to change page because
        // only the anchor part of the URL is changed. (I.e. when we're on a
        // autodiscovery page.)
        if (window.top.location.href.indexOf(urlPath + "#") >= 0) {
            msg = msg.replace(urlPath, urlPath + "?_=1");
        }
        return $.notify(msg, {"nag": true});
    }

    notifyScheduledResults();
});


